package com.tjise.asked.http;

import java.util.concurrent.TimeUnit;

import okhttp3.ConnectionPool;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import retrofit2.CallAdapter;
import retrofit2.Converter;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava3.RxJava3CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

public class RetrofitClient {
    public static final String baseUrl = "https://syjhs-443.webvpn.tjise.edu.cn/";
    private static final int TIMEOUT_CONNECT = 10;// 连接超时时间（秒）
    private static final int TIMEOUT_READ = 30;// 读取超时时间（秒）
    private static final int TIMEOUT_WRITE = 30; // 写入超时时间（秒）
    private static final int MAX_IDLE_CONNECTIONS = 8; // 最大空闲连接数
    private static final int KEEP_ALIVE_DURATION = 30;// 空闲连接保持时间（秒）
    private static final TimeUnit KEEP_ALIVE_TIME_UNIT = TimeUnit.SECONDS; // 空闲连接保持时间单位
    private static RetrofitClient retrofitClient;

    private ApiService apiService;

    private OkHttpClient okHttpClient;

    private Retrofit retrofit;

    public static RetrofitClient getInstance() {
        if (retrofitClient == null) {
            retrofitClient = new RetrofitClient();
        }
        return retrofitClient;
    }

    private Retrofit getRetrofit() {

        if (retrofit == null) {
            retrofit = new Retrofit.Builder()
                    .baseUrl(baseUrl)
                    .client(getOkHttpClient())
                    .addConverterFactory(GsonConverterFactory.create())
                    .addCallAdapterFactory(RxJava3CallAdapterFactory.create())
                    .build();
        }
        return retrofit;
    }

    public ApiService getApiService() {
        if (this.apiService == null)
            this.apiService = (ApiService) getRetrofit().create(ApiService.class);
        return this.apiService;
    }

    public OkHttpClient getOkHttpClient() {
        if (okHttpClient == null) {
            //使用连接池，尽量保持TCP连接的复用，而不是立即关闭。
            ConnectionPool connectionPool = new ConnectionPool(MAX_IDLE_CONNECTIONS, KEEP_ALIVE_DURATION, KEEP_ALIVE_TIME_UNIT);
            okHttpClient = new OkHttpClient.Builder()
                    .connectTimeout(TIMEOUT_CONNECT, TimeUnit.SECONDS)
                    .readTimeout(TIMEOUT_READ, TimeUnit.SECONDS)
                    .writeTimeout(TIMEOUT_WRITE, TimeUnit.SECONDS)
                    .connectionPool(connectionPool)
                    .build();
        }
        return okHttpClient;
    }
}
